@@ -1,8 +1,19 @@
+"""
+Software injection into pre-existing data files
+===============================================
+
+Add a software injection into a set of SFTs.
+
+In this case, the set of SFTs is generated using Makefakedata_v5,
+but the same procedure can be applied to any other set of SFTs
+(including real detector data).
+"""
+
 import os
 import numpy as np
 import pyfstat
 
-label = os.path.splitext(os.path.basename(__file__))[0]
+label = "PyFstat_example_injection_into_noise_sfts"
 outdir = os.path.join("PyFstat_example_data", label)
 
 tstart = 1269734418
@@ -52,8 +63,6 @@ coherent_search = pyfstat.ComputeFstat(
     maxCoverFreq=-0.5,
 )
 FS_1 = coherent_search.get_fullycoherent_twoF(
-    noise_and_signal_writer.tstart,
-    noise_and_signal_writer.tend,
     noise_and_signal_writer.F0,
     noise_and_signal_writer.F1,
     noise_and_signal_writer.F2,
@@ -115,8 +124,6 @@ coherent_search = pyfstat.ComputeFstat(
     maxCoverFreq=-0.5,
 )
 FS_2 = coherent_search.get_fullycoherent_twoF(
-    add_signal_writer.tstart,
-    add_signal_writer.tend,
     add_signal_writer.F0,
     add_signal_writer.F1,
     add_signal_writer.F2,
