@@ -555,8 +555,8 @@ class TestGlitchWriter(TestWriter):
         max_freq_noglitch = freqs_noglitch[np.argmax(data_noglitch, axis=0)]
         max_freq_glitch = freqs_glitch[np.argmax(data_glitch, axis=0)]
         print([max_freq_vanilla, max_freq_noglitch, max_freq_glitch])
-        self.assertEqual(times_noglitch, times_vanilla)
-        self.assertEqual(times_glitch, times_vanilla)
+        self.assertTrue(np.all(times_noglitch == times_vanilla))
+        self.assertTrue(np.all(times_glitch == times_vanilla))
         self.assertEqual(len(np.unique(max_freq_vanilla)), 1)
         self.assertEqual(len(np.unique(max_freq_noglitch)), 1)
         self.assertEqual(len(np.unique(max_freq_glitch)), 2)
@@ -663,8 +663,6 @@ class TestComputeFstat(BaseForTestsWithData):
             maxCoverFreq=self.F0 + 0.1,
         )
         FS = search.get_fullycoherent_twoF(
-            tstart=self.tstart,
-            tend=self.tstart + self.duration,
             F0=self.F0,
             F1=self.F1,
             F2=self.F2,
@@ -683,13 +681,11 @@ class TestComputeFstat(BaseForTestsWithData):
             search_ranges=self.search_ranges,
         )
         twoF = search.get_fullycoherent_twoF(
-            self.Writer.tstart,
-            self.Writer.tend(),
-            self.Writer.F0,
-            self.Writer.F1,
-            self.Writer.F2,
-            self.Writer.Alpha,
-            self.Writer.Delta,
+            F0=self.Writer.F0,
+            F1=self.Writer.F1,
+            F2=self.Writer.F2,
+            Alpha=self.Writer.Alpha,
+            Delta=self.Writer.Delta,
         )
         diff = np.abs(twoF - twoF_predicted) / twoF_predicted
         print(
@@ -713,13 +709,11 @@ class TestComputeFstat(BaseForTestsWithData):
             search_ranges=self.search_ranges,
         )
         twoF2 = search.get_fullycoherent_twoF(
-            self.Writer.tstart,
-            self.Writer.tend(),
-            self.Writer.F0,
-            self.Writer.F1,
-            self.Writer.F2,
-            self.Writer.Alpha,
-            self.Writer.Delta,
+            F0=self.Writer.F0,
+            F1=self.Writer.F1,
+            F2=self.Writer.F2,
+            Alpha=self.Writer.Alpha,
+            Delta=self.Writer.Delta,
         )
         diff = np.abs(twoF2 - twoF_predicted) / twoF_predicted
         print(
@@ -749,13 +743,11 @@ class TestComputeFstat(BaseForTestsWithData):
             detectors=self.Writer.detectors,
         )
         FS_from_file = search.get_fullycoherent_twoF(
-            self.Writer.tstart,
-            self.Writer.tend(),
-            self.Writer.F0,
-            self.Writer.F1,
-            self.Writer.F2,
-            self.Writer.Alpha,
-            self.Writer.Delta,
+            F0=self.Writer.F0,
+            F1=self.Writer.F1,
+            F2=self.Writer.F2,
+            Alpha=self.Writer.Alpha,
+            Delta=self.Writer.Delta,
         )
         self.assertTrue(np.abs(predicted_FS - FS_from_file) / FS_from_file < 0.3)
 
@@ -775,18 +767,16 @@ class TestComputeFstat(BaseForTestsWithData):
             detectors=self.Writer.detectors,
         )
         FS_from_dict = search.get_fullycoherent_twoF(
-            self.Writer.tstart,
-            self.Writer.tend(),
-            self.Writer.F0,
-            self.Writer.F1,
-            self.Writer.F2,
-            self.Writer.Alpha,
-            self.Writer.Delta,
+            F0=self.Writer.F0,
+            F1=self.Writer.F1,
+            F2=self.Writer.F2,
+            Alpha=self.Writer.Alpha,
+            Delta=self.Writer.Delta,
         )
         self.assertTrue(FS_from_dict == FS_from_file)
 
     def test_get_fully_coherent_BSGL(self):
-        # first pure noise, expect lnBSGL<0
+        # first pure noise, expect log10BSGL<0
         search_H1L1 = pyfstat.ComputeFstat(
             tref=self.tref,
             minStartTime=self.tstart,
@@ -797,17 +787,15 @@ class TestComputeFstat(BaseForTestsWithData):
             maxCoverFreq=self.F0 + 0.1,
             BSGL=True,
         )
-        lnBSGL = search_H1L1.get_fullycoherent_twoF(
-            tstart=self.tstart,
-            tend=self.tstart + self.duration,
+        log10BSGL = search_H1L1.get_fullycoherent_twoF(
             F0=self.F0,
             F1=self.F1,
             F2=self.F2,
             Alpha=self.Alpha,
             Delta=self.Delta,
         )
-        self.assertTrue(lnBSGL < 0)
-        # now with an added signal, expect lnBSGL>0
+        self.assertTrue(log10BSGL < 0)
+        # now with an added signal, expect log10BSGL>0
         search_H1L1 = pyfstat.ComputeFstat(
             tref=self.tref,
             minStartTime=self.tstart,
@@ -828,16 +816,14 @@ class TestComputeFstat(BaseForTestsWithData):
             maxCoverFreq=self.F0 + 0.1,
             BSGL=True,
         )
-        lnBSGL = search_H1L1.get_fullycoherent_twoF(
-            tstart=self.tstart,
-            tend=self.tstart + self.duration,
+        log10BSGL = search_H1L1.get_fullycoherent_twoF(
             F0=self.F0,
             F1=self.F1,
             F2=self.F2,
             Alpha=self.Alpha,
             Delta=self.Delta,
         )
-        self.assertTrue(lnBSGL > 0)
+        self.assertTrue(log10BSGL > 0)
 
     def test_cumulative_twoF(self):
         Nsft = 100
@@ -861,13 +847,13 @@ class TestComputeFstat(BaseForTestsWithData):
             num_segments=Nsft + 1,
         )
         twoF = search.get_fullycoherent_twoF(
-            self.Writer.tstart,
-            self.Writer.tstart + taus[-1],
-            self.Writer.F0,
-            self.Writer.F1,
-            self.Writer.F2,
-            self.Writer.Alpha,
-            self.Writer.Delta,
+            F0=self.Writer.F0,
+            F1=self.Writer.F1,
+            F2=self.Writer.F2,
+            Alpha=self.Writer.Alpha,
+            Delta=self.Writer.Delta,
+            tstart=self.Writer.tstart,
+            tend=self.Writer.tstart + taus[-1],
         )
         reldiff = np.abs(twoF_cumulative[-1] - twoF) / twoF
         print(
@@ -904,7 +890,6 @@ class TestComputeFstatNoNoise(BaseForTestsWithData):
     def test_run_computefstatistic_single_point_no_noise(self):
 
         predicted_FS = self.Writer.predict_fstat(assumeSqrtSX=1)
-        shutil.copy(self.Writer.sftfilepath, ".")
         search = pyfstat.ComputeFstat(
             tref=self.Writer.tref,
             assumeSqrtSX=1,
@@ -912,13 +897,11 @@ class TestComputeFstatNoNoise(BaseForTestsWithData):
             search_ranges=self.search_ranges,
         )
         FS = search.get_fullycoherent_twoF(
-            self.Writer.tstart,
-            self.Writer.tend(),
-            self.Writer.F0,
-            self.Writer.F1,
-            self.Writer.F2,
-            self.Writer.Alpha,
-            self.Writer.Delta,
+            F0=self.Writer.F0,
+            F1=self.Writer.F1,
+            F2=self.Writer.F2,
+            Alpha=self.Writer.Alpha,
+            Delta=self.Writer.Delta,
         )
         self.assertTrue(np.abs(predicted_FS - FS) / FS < 0.3)
 
@@ -942,13 +925,11 @@ class TestComputeFstatNoNoise(BaseForTestsWithData):
             search_ranges=self.search_ranges,
         )
         FS = search.get_fullycoherent_twoF(
-            self.Writer.tstart,
-            self.Writer.tend(),
-            self.Writer.F0,
-            self.Writer.F1,
-            self.Writer.F2,
-            self.Writer.Alpha,
-            self.Writer.Delta,
+            F0=self.Writer.F0,
+            F1=self.Writer.F1,
+            F2=self.Writer.F2,
+            Alpha=self.Writer.Alpha,
+            Delta=self.Writer.Delta,
         )
         self.assertTrue(np.abs(predicted_FS - FS) / FS < 0.3)
 
@@ -1248,14 +1229,14 @@ class BaseForMCMCSearchTests(BaseForTestsWithData):
                 "transient_duration": self.Writer.transientTau,
             }
 
-        for k in inj.keys():
+        for k in self.max_dict.keys():
             reldiff = np.abs((self.max_dict[k] - inj[k]) / inj[k])
             print("max2F  {:s} reldiff: {:.2e}".format(k, reldiff))
             reldiff = np.abs((summary_stats[k]["mean"] - inj[k]) / inj[k])
             print("mean   {:s} reldiff: {:.2e}".format(k, reldiff))
             reldiff = np.abs((summary_stats[k]["median"] - inj[k]) / inj[k])
             print("median {:s} reldiff: {:.2e}".format(k, reldiff))
-        for k in inj.keys():
+        for k in self.max_dict.keys():
             lower = summary_stats[k]["mean"] - nsigmas * summary_stats[k]["std"]
             upper = summary_stats[k]["mean"] + nsigmas * summary_stats[k]["std"]
             within = (inj[k] >= lower) and (inj[k] <= upper)
@@ -1490,8 +1471,8 @@ class TestMCMCTransientSearch(BaseForMCMCSearchTests):
 
     def setup_method(self, method):
         self.transientWindowType = "rect"
-        self.transientStartTime = self.tstart + 0.25 * self.duration
-        self.transientTau = 0.5 * self.duration
+        self.transientStartTime = int(self.tstart + 0.25 * self.duration)
+        self.transientTau = int(0.5 * self.duration)
         self.Writer = pyfstat.Writer(
             label=self.label,
             tstart=self.tstart,
@@ -1510,15 +1491,74 @@ class TestMCMCTransientSearch(BaseForMCMCSearchTests):
             transientTau=self.transientTau,
         )
         self.Writer.make_data(verbose=True)
-
-    def test_transient_MCMC(self):
-
-        theta = {
+        self.basic_theta = {
             "F0": self.F0,
             "F1": self.F1,
             "F2": self.F2,
             "Alpha": self.Alpha,
             "Delta": self.Delta,
+        }
+        self.MCMC_params = {
+            "nsteps": [50, 50],
+            "nwalkers": 50,
+            "ntemps": 2,
+            "log10beta_min": -1,
+        }
+
+    def test_transient_MCMC_t0only(self):
+
+        theta = {
+            **self.basic_theta,
+            "transient_tstart": {
+                "type": "unif",
+                "lower": self.Writer.tstart,
+                "upper": self.Writer.tend() - 2 * self.Writer.Tsft,
+            },
+            "transient_duration": self.transientTau,
+        }
+        self.search = pyfstat.MCMCTransientSearch(
+            label=self.label,
+            outdir=self.outdir,
+            theta_prior=theta,
+            tref=self.tref,
+            sftfilepattern=self.Writer.sftfilepath,
+            **self.MCMC_params,
+            transientWindowType=self.transientWindowType,
+        )
+        self.search.run(plot_walkers=False)
+        self.search.print_summary()
+        self._check_twoF_predicted()
+        self._check_mcmc_quantiles(transient=True)
+
+    def test_transient_MCMC_tauonly(self):
+
+        theta = {
+            **self.basic_theta,
+            "transient_tstart": self.transientStartTime,
+            "transient_duration": {
+                "type": "unif",
+                "lower": 2 * self.Writer.Tsft,
+                "upper": self.Writer.duration - 2 * self.Writer.Tsft,
+            },
+        }
+        self.search = pyfstat.MCMCTransientSearch(
+            label=self.label,
+            outdir=self.outdir,
+            theta_prior=theta,
+            tref=self.tref,
+            sftfilepattern=self.Writer.sftfilepath,
+            **self.MCMC_params,
+            transientWindowType=self.transientWindowType,
+        )
+        self.search.run(plot_walkers=False)
+        self.search.print_summary()
+        self._check_twoF_predicted()
+        self._check_mcmc_quantiles(transient=True)
+
+    def test_transient_MCMC_t0_tau(self):
+
+        theta = {
+            **self.basic_theta,
             "transient_tstart": {
                 "type": "unif",
                 "lower": self.Writer.tstart,
@@ -1536,10 +1576,7 @@ class TestMCMCTransientSearch(BaseForMCMCSearchTests):
             theta_prior=theta,
             tref=self.tref,
             sftfilepattern=self.Writer.sftfilepath,
-            nsteps=[100, 100],
-            nwalkers=100,
-            ntemps=2,
-            log10beta_min=-1,
+            **self.MCMC_params,
             transientWindowType=self.transientWindowType,
         )
         self.search.run(plot_walkers=False)
@@ -1572,9 +1609,7 @@ class TestGridSearch(BaseForTestsWithData):
         search.run()
         self.assertTrue(os.path.isfile(search.out_file))
         max2F_point = search.get_max_twoF()
-        self.assertTrue(
-            np.all(max2F_point["twoF"] >= search.data[:, search.keys.index("twoF")])
-        )
+        self.assertTrue(np.all(max2F_point["twoF"] >= search.data["twoF"]))
 
     def test_grid_search_against_CFSv2(self):
         search = pyfstat.GridSearch(
@@ -1647,23 +1682,6 @@ class TestGridSearch(BaseForTestsWithData):
         search.run()
         self.assertTrue(os.path.isfile(search.out_file))
 
-    def test_slice_grid_search(self):
-        # FIXME this one doesn't check the results at all yet
-        search = pyfstat.SliceGridSearch(
-            "slice_grid_search",
-            self.outdir,
-            self.Writer.sftfilepath,
-            F0s=self.F0s,
-            F1s=self.F1s,
-            F2s=[self.Writer.F2],
-            Alphas=[self.Writer.Alpha],
-            Deltas=[self.Writer.Delta],
-            tref=self.tref,
-            Lambda0=[30.0, 0.0, 0.0, 0.0],
-        )
-        fig, axes = search.run(save=False)
-        self.assertTrue(fig is not None)
-
     def test_glitch_grid_search(self):
         search = pyfstat.GridGlitchSearch(
             "grid_grid_search",
@@ -1680,24 +1698,6 @@ class TestGridSearch(BaseForTestsWithData):
         search.run()
         self.assertTrue(os.path.isfile(search.out_file))
 
-    def test_sliding_window(self):
-        # FIXME this one doesn't check the results at all yet
-        search = pyfstat.FrequencySlidingWindow(
-            "grid_grid_search",
-            self.outdir,
-            self.Writer.sftfilepath,
-            F0s=self.F0s,
-            F1=self.Writer.F1,
-            F2=self.Writer.F2,
-            Alpha=self.Writer.Alpha,
-            Delta=self.Writer.Delta,
-            tref=self.tref,
-            minStartTime=self.Writer.tstart,
-            maxStartTime=self.Writer.tend(),
-        )
-        search.run()
-        self.assertTrue(os.path.isfile(search.out_file))
-
 
 class TestTransientGridSearch(BaseForTestsWithData):
     label = "TestTransientGridSearch"
@@ -1726,19 +1726,8 @@ class TestTransientGridSearch(BaseForTestsWithData):
         search.run()
         self.assertTrue(os.path.isfile(search.out_file))
         max2F_point = search.get_max_twoF()
-        self.assertTrue(
-            np.all(max2F_point["twoF"] >= search.data[:, search.keys.index("twoF")])
-        )
-        tCWfile = (
-            search.tCWfilebase
-            + "{:.16f}_{:.16f}_{:.16f}_{:.16g}_{:.16g}.dat".format(
-                max2F_point["F0"],
-                max2F_point["Alpha"],
-                max2F_point["Delta"],
-                max2F_point["F1"],
-                max2F_point["F2"],
-            )
-        )
+        self.assertTrue(np.all(max2F_point["twoF"] >= search.data["twoF"]))
+        tCWfile = search.get_transient_fstat_map_filename(max2F_point)
         tCW_out = pyfstat.helper_functions.read_txt_file_with_header(
             tCWfile, comments="#"
         )
